package com.blazinggames.cqfs.converse
{
	// Adobe Imports
	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	
	/**
	 * The dialog window that is used for cartoon style dialog sequences. The
	 * movie clip that implements this class should have 12 labels for the 6 
	 * different dialog state named:
	 * description and descriptionDone
	 * speach and speachDone
	 * thought and thoughtDone
	 * exclamation and exclamationDone
	 * whisper and whisperDone
	 * robotic and roboticDone
	 *
	 * the class should also have a text area for the text that is named
	 * _dialog_txt and a button for quick text named _quick_btn.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007 Blazing Games Inc. All Rights Reserved.
	*/
	public class CartoonSpeach
	extends MovieClip
	implements ISpeach
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public static const PREVIOUS_CODE = -1;
		public static const NEXT_CODE = -2;

		protected static const DEFAULT_PREV:String = "Previous choices...";
		protected static const DEFAULT_NEXT:String = "More choices...";

// *********************
// ***** VARIABLES *****
// *********************

		protected var _responses:Array;
		protected var _dialogText:String;
		protected var _dialogQuick:Boolean;
		protected var _dialogType:int;
		protected var _responseData:Array;
		protected var _lastResponse:int = 0;

		// handling of more than 6 messages
		protected var _prevText:String = DEFAULT_PREV;
		protected var _prevType:int = 0;
		protected var _nextText:String = DEFAULT_NEXT;
		protected var _nextType:int = 0;
		protected var _topChoice:int;

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function CartoonSpeach()
		{
			_responses = new Array(
				_response0_movie, _response1_movie, _response2_movie,
				_response3_movie, _response4_movie, _response5_movie
			);
			_responseData = new Array();
			
			for (var cntr:int = 0; cntr < _responses.length; ++cntr)
				_responses[cntr].addEventListener(MouseEvent.CLICK, clickHandler);
		}

// ****************************************
// ***** Functions - ISpeach required *****
// ****************************************

		public function clearDisplay():void
		{
			while (_responseData.length > 0)
				_responseData.pop();
			_dialog_movie.hideDialog();
			for (var cntr:int = 0; cntr < _responses.length; ++cntr) {
				_responses[cntr].hideResponse();
			}
		}
		
		public function setDialog(s:String, quickly:Boolean=false, dtype:int=0):void
		{
			_dialogText = s;
			_dialogQuick = quickly;
			_dialogType = dtype;
		}
		
		public function addResponse(s:String, id:int, dtype:int=0):void
		{
			var obj:Object = {str:s, rcode:id, rtype:dtype};
			
			_responseData.push(obj);
		}
		
		public function startDisplay():void
		{
			_dialog_movie.showDialog(_dialogText, _dialogQuick,	_dialogType);
			_topChoice = 0;
			updateResponses();
		}

		public function getLastResponse():int
		{
			return _lastResponse;
		}

// **************************************
// ***** Functions - Event handlers *****
// **************************************

		protected function clickHandler(evt:MouseEvent)
		{
			_lastResponse = evt.currentTarget.getResultCode();
			if (_lastResponse >= 0) {
				var evnt:Event = new Event(Converse.SPEACH_EVENT);
				this.dispatchEvent(evnt);
			} else {
				if (_lastResponse == PREVIOUS_CODE) {
					_topChoice -= 4;
					if (_topChoice <= 1)
						_topChoice = 0;
				} else {
					_topChoice += 4;
					if (_topChoice == 4)
						_topChoice = 5;
				}
				updateResponses();
			}
		}
/*
		protected function finishedShowing()
		{
			stop();
			var evnt:Event = new Event(Converse.RESPONSE_SHOWN_EVENT);
			this.dispatchEvent(evnt);
			_animatedText.setText(_message_txt, messageText, 1, 24);
		}
*/

		protected function updateResponses()
		{
			var obj:Object;
			var cntr:int;
			var remaining:int = _responseData.length - _topChoice;
			var maxShow:int = 6;
			var responseIndex:int = 0;

			for (cntr = 0; cntr < _responses.length; ++cntr)
				_responses[cntr].hideResponse();

			if (_topChoice > 0) {
				--maxShow;
				++responseIndex;
				_responses[0].showResponse(_prevText,
						PREVIOUS_CODE, true, _prevType);
			}
			if (remaining > maxShow) {
				--maxShow;
				_responses[5].showResponse(_nextText,
						NEXT_CODE, true, _nextType);
			} else
				maxShow = remaining;
			for (cntr = 0; cntr < maxShow; ++cntr) {
				obj = _responseData[_topChoice + cntr];
				_responses[responseIndex].showResponse(obj.str,obj.rcode,false,obj.rtype);
				++responseIndex;
			}
		}

// *************************************
// ***** Functions - Setup options *****
// *************************************

		public function setPreviousMessage(str:String, rtype:int = 0):void
		{
			_prevText = str;
			_prevType = rtype;
		}

		public function setNextMessage(str:String, rtype:int):void
		{
			_nextText = str;
			_nextType = rtype;
		}
	} // end class
} // end package

