﻿package com.blazinggames.cqfs.converse
{
	
	/**
	 * The interface for the scripting language support of converse script.
	 * The idea here is that scripting is handled by command strings and querry
	 * strings. The program using scripting can override the default scripting
	 * support to support it's own special (game specific) quarries and commands
	 * but the default support should be good enough for most conversations.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007 Blazing Games Inc. All Rights Reserved.
	*/
	public class DefaultScriptCommand
	implements IScriptCommand
	{
// *********************
// ***** CONSTANTS *****
// *********************

		// FLAG Constants
		public const FLAG_TRUE = "TRUE";
		public const FLAG_FALSE = "FALSE";
		
		// Commands
		public const CMD_INVALID:int = -1;
		public const CMD_SETFLAG:int = 0;
		public const CMD_CLEARFLAG:int = 1;
		public const CMD_TOGGLEFLAG:int = 2;
		public const COMMAND_STRINGS:Array =  new Array(
			"SETFLAG", "CLEARFLAG", "TOGGLEFLAG"
		);

		// Queries
		public const QUERY_ISFLAGSET:int = 0;
		public const QUERY_ISFLAGCLEAR:int = 1;
		public const QUERY_STRINGS:Array = new Array(
			"ISFLAGSET", "ISFLAGCLEAR"
		);

// *********************
// ***** VARIABLES *****
// *********************

		protected var _debug:Boolean;
		protected var _flags:Object;

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function DefaultScriptCommand(d:Boolean = false)
		{
			_debug = d;
			_flags = new Object();
			_flags[FLAG_TRUE] = true;
			_flags[FLAG_FALSE] = false;
		}

// ***********************************************
// ***** Functions - IScriptCommand required *****
// ***********************************************

		public function parseCommand(s:String):void
		{
			var cntr:int;
			var tokens:Array = s.split(/\s\s*/);
			var command:String = tokens[0].toUpperCase();
			var commandID:int = CMD_INVALID;
			
			if (_debug)
				for (cntr; cntr < tokens.length; ++cntr)
					trace("Token " + cntr + " =\"" + tokens[cntr] + "\"");
			// first, see if known command
			for (cntr = 0; cntr < COMMAND_STRINGS.length; ++cntr)
				if (COMMAND_STRINGS[cntr] == command)
					commandID = cntr;
			// then process the command
			switch (commandID) {
				case CMD_SETFLAG:
					if (_debug) 
						trace (tokens[1] + " changed from " +
								_flags[tokens[1]] + " to true");
					_flags[tokens[1]] = true;
					break;

				case CMD_CLEARFLAG:
					if (_debug) 
						trace (tokens[1] + " changed from " +
								_flags[tokens[1]] + " to false");
					_flags[tokens[1]] = false;
					break;

				case CMD_TOGGLEFLAG:
					if (_debug) 
						trace (tokens[1] + " changed from " +
								_flags[tokens[1]] + " to " + !_flags[tokens[1]]);
					_flags[tokens[1]] = !_flags[tokens[1]];
					break;

				case CMD_INVALID:
				default:
					trace("Unknown or invalid command: " + s)
			}
		}

		public function parseQuery(s:String):Boolean
		{
			var cntr:int;
			var tokens:Array = s.split(/\s\s*/);
			var command:String = tokens[0].toUpperCase();
			var commandID:int = CMD_INVALID;
			var ret:Boolean = false;
			
			if (_debug)
				for (cntr; cntr < tokens.length; ++cntr)
					trace("Token " + cntr + " =\"" + tokens[cntr] + "\"");
			// first, see if known querry
			for (cntr = 0; cntr < QUERY_STRINGS.length; ++cntr)
				if (QUERY_STRINGS[cntr] == command)
					commandID = cntr;
			// then process the querry
			switch (commandID) {
				case QUERY_ISFLAGSET:
					ret = (_flags[tokens[1]] == true);
					break;

				case QUERY_ISFLAGCLEAR:
					ret = (_flags[tokens[1]] == false);
					break;

				case CMD_INVALID:
				default:
					trace("Unknown or invalid command: " + s)
			}
			
			return ret;
		}

	} // end Class
} // end package

